@@ -16,7 +16,7 @@ use crate::util::{Pool, PoolClone, PoolDefault, PoolRef};
 use self::Insert::*;
 use self::InsertAction::*;
 
-const NODE_SIZE: usize = NodeSize::USIZE;
+pub(crate) const NODE_SIZE: usize = NodeSize::USIZE;
 const MEDIAN: usize = (NODE_SIZE + 1) >> 1;
 
 pub trait BTreeValue {
@@ -37,6 +37,13 @@ pub trait BTreeValue {
     fn cmp_values(&self, other: &Self) -> Ordering;
 }
 
+/// A node in a `BTree`.
+///
+/// A node is either internal, or a leaf. Leaf nodes have `None` for every child, and internal
+/// nodes have `Some(_)` for every child. There will never be a mixture of `None`s and `Some`s.
+///
+/// The `children` array is never empty, and always has exactly one more element than `keys`. The
+/// empty tree has no keys, and a single `None` child.
 pub(crate) struct Node<A> {
     keys: Chunk<A, NodeSize>,
     children: Chunk<Option<PoolRef<Node<A>>>, Add1<NodeSize>>,
@@ -86,15 +93,25 @@ enum InsertAction<A> {
     InsertSplit(Node<A>, A, Node<A>),
 }
 
+/// The result of a remove operation.
 pub(crate) enum Remove<A> {
+    /// The key to remove was not found in the tree; nothing changed.
     NoChange,
+    /// The key was found and removed: here it is.
     Removed(A),
+    /// The key was found, and the root node of the tree was modified: here is the found key, and
+    /// the new root node.
     Update(A, Node<A>),
 }
 
+enum Boundary {
+    Lowest,
+    Highest,
+}
+
 enum RemoveAction {
     DeleteAt(usize),
-    PullUp(usize, usize, usize),
+    PullUp(Boundary, usize, usize),
     Merge(usize),
     StealFromLeft(usize),
     StealFromRight(usize),
@@ -129,6 +146,8 @@ impl<A> Node<A> {
         self.keys.len() < NODE_SIZE
     }
 
+    /// This name is slightly misleading, because we actually check whether this node is the
+    /// minimum allowed size (for a non-root node).
     #[inline]
     fn too_small(&self) -> bool {
         self.keys.len() < MEDIAN
@@ -174,6 +193,78 @@ impl<A> Node<A> {
 }
 
 impl<A: BTreeValue> Node<A> {
+    // Checks that this tree is balanced, and returns its depth.
+    #[cfg(test)]
+    pub(crate) fn check_depth(&self) -> usize {
+        if self.children.is_empty() {
+            // This is an empty tree.
+            0
+        } else if self.children[0].is_none() {
+            // This is a leaf node.
+            1
+        } else {
+            let mut depth = None;
+            for c in self.children.iter() {
+                let d = c.as_ref().unwrap().check_depth();
+                assert!(depth.is_none() || depth == Some(d));
+                depth = Some(d);
+            }
+            depth.unwrap()
+        }
+    }
+
+    // Checks that the keys are in the right order.
+    #[cfg(test)]
+    pub(crate) fn check_order(&self) {
+        fn recurse<A: BTreeValue>(node: &Node<A>) -> (&A, &A) {
+            for window in node.keys.windows(2) {
+                assert!(window[0].cmp_values(&window[1]) == Ordering::Less);
+            }
+            if node.is_leaf() {
+                (node.keys.first().unwrap(), node.keys.last().unwrap())
+            } else {
+                for i in 0..node.keys.len() {
+                    let left_max = recurse(node.children[i].as_ref().unwrap()).1;
+                    let right_min = recurse(node.children[i + 1].as_ref().unwrap()).0;
+                    assert!(node.keys[i].cmp_values(left_max) == Ordering::Greater);
+                    assert!(node.keys[i].cmp_values(right_min) == Ordering::Less);
+                }
+                (
+                    recurse(node.children.first().unwrap().as_ref().unwrap()).0,
+                    recurse(node.children.last().unwrap().as_ref().unwrap()).1,
+                )
+            }
+        }
+        if !self.keys.is_empty() {
+            recurse(self);
+        }
+    }
+
+    #[cfg(test)]
+    pub(crate) fn check_size(&self) {
+        fn recurse<A: BTreeValue>(node: &Node<A>) {
+            assert!(node.keys.len() + 1 == node.children.len());
+            assert!(node.keys.len() + 1 >= MEDIAN);
+            if !node.is_leaf() {
+                for c in &node.children {
+                    recurse(c.as_ref().unwrap());
+                }
+            }
+        }
+        if !self.is_leaf() {
+            for c in &self.children {
+                recurse(c.as_ref().unwrap());
+            }
+        }
+    }
+
+    #[cfg(test)]
+    pub(crate) fn check_sane(&self) {
+        self.check_depth();
+        self.check_order();
+        self.check_size();
+    }
+
     fn child_contains<BK>(&self, index: usize, key: &BK) -> bool
     where
         BK: Ord + ?Sized,
@@ -240,14 +331,13 @@ impl<A: BTreeValue> Node<A> {
         }
         match A::search_key(&self.keys, key) {
             Ok(index) => Some(&self.keys[index]),
-            Err(index) => match self.children[index] {
-                None if index == 0 => None,
-                None => match self.keys.get(index - 1) {
-                    Some(_) => Some(&self.keys[index - 1]),
-                    None => None,
-                },
-                Some(ref node) => node.lookup_prev(key),
-            },
+            Err(index) => self.children[index]
+                .as_ref()
+                .and_then(|node| node.lookup_prev(key))
+                // If we haven't found our search key yet, it isn't in any child subtree of ours.
+                // That means that if index == 0 then we have no predecessor for the search key,
+                // and if index > 0 then the predecessor is our key at index - 1.
+                .or_else(|| index.checked_sub(1).and_then(|i| self.keys.get(i))),
         }
     }
 
@@ -261,13 +351,12 @@ impl<A: BTreeValue> Node<A> {
         }
         match A::search_key(&self.keys, key) {
             Ok(index) => Some(&self.keys[index]),
-            Err(index) => match self.children[index] {
-                None => match self.keys.get(index) {
-                    Some(_) => Some(&self.keys[index]),
-                    None => None,
-                },
-                Some(ref node) => node.lookup_next(key),
-            },
+            Err(index) => self.children[index]
+                .as_ref()
+                .and_then(|node| node.lookup_next(key))
+                // If we don't find the search key in the child subtree, then either our next key
+                // is the search key's successor, or else we don't have a successor in our subtree.
+                .or_else(|| self.keys.get(index)),
         }
     }
 
@@ -284,16 +373,13 @@ impl<A: BTreeValue> Node<A> {
         if self.keys.is_empty() {
             return None;
         }
-        match A::search_key(&self.keys, key) {
-            Ok(index) => Some(&mut self.keys[index]),
-            Err(index) => match self.children[index] {
-                None if index == 0 => None,
-                None => match self.keys.get(index - 1) {
-                    Some(_) => Some(&mut self.keys[index - 1]),
-                    None => None,
-                },
-                Some(ref mut node) => PoolRef::make_mut(pool, node).lookup_prev_mut(pool, key),
-            },
+        let keys = &mut self.keys;
+        match A::search_key(keys, key) {
+            Ok(index) => Some(&mut keys[index]),
+            Err(index) => self.children[index]
+                .as_mut()
+                .and_then(|node| PoolRef::make_mut(pool, node).lookup_prev_mut(pool, key))
+                .or_else(|| index.checked_sub(1).and_then(move |i| keys.get_mut(i))),
         }
     }
 
@@ -310,15 +396,13 @@ impl<A: BTreeValue> Node<A> {
         if self.keys.is_empty() {
             return None;
         }
-        match A::search_key(&self.keys, key) {
-            Ok(index) => Some(&mut self.keys[index]),
-            Err(index) => match self.children[index] {
-                None => match self.keys.get(index) {
-                    Some(_) => Some(&mut self.keys[index]),
-                    None => None,
-                },
-                Some(ref mut node) => PoolRef::make_mut(pool, node).lookup_next_mut(pool, key),
-            },
+        let keys = &mut self.keys;
+        match A::search_key(keys, key) {
+            Ok(index) => Some(&mut keys[index]),
+            Err(index) => self.children[index]
+                .as_mut()
+                .and_then(|node| PoolRef::make_mut(pool, node).lookup_next_mut(pool, key))
+                .or_else(move || keys.get_mut(index)),
         }
     }
 
@@ -395,7 +479,17 @@ impl<A: BTreeValue> Node<A> {
                         path.push((self, index));
                         path
                     }
-                    None => Vec::new(),
+                    None => {
+                        // go back up to find next
+                        while let Some((node, idx)) = path.last() {
+                            if node.keys.len() == *idx {
+                                path.pop();
+                            } else {
+                                break;
+                            }
+                        }
+                        path
+                    }
                 },
                 Some(ref node) => {
                     path.push((self, index));
@@ -424,14 +518,22 @@ impl<A: BTreeValue> Node<A> {
                 path
             }
             Err(index) => match self.children[index] {
-                None if index == 0 => Vec::new(),
-                None => match self.keys.get(index - 1) {
-                    Some(_) => {
-                        path.push((self, index - 1));
-                        path
+                None if index == 0 => {
+                    // go back up to find prev
+                    while let Some((_, idx)) = path.last_mut() {
+                        if *idx == 0 {
+                            path.pop();
+                        } else {
+                            *idx -= 1;
+                            break;
+                        }
                     }
-                    None => Vec::new(),
-                },
+                    path
+                }
+                None => {
+                    path.push((self, index - 1));
+                    path
+                }
                 Some(ref node) => {
                     path.push((self, index));
                     node.path_prev(key, path)
@@ -550,6 +652,11 @@ impl<A: BTreeValue> Node<A> {
         self.children.push_back(child);
     }
 
+    fn is_leaf(&self) -> bool {
+        // `children` is never empty, so we can index it.
+        self.children[0].is_none()
+    }
+
     pub(crate) fn insert(&mut self, pool: &Pool<Node<A>>, value: A) -> Insert<A>
     where
         A: Clone,
@@ -618,14 +725,32 @@ impl<A: BTreeValue> Node<A> {
         A::Key: Borrow<BK>,
     {
         let index = A::search_key(&self.keys, key);
-        self.remove_index(pool, index, key)
+        self.remove_index(pool, index, Ok(key))
+    }
+
+    fn remove_target<BK>(
+        &mut self,
+        pool: &Pool<Node<A>>,
+        target: Result<&BK, Boundary>,
+    ) -> Remove<A>
+    where
+        A: Clone,
+        BK: Ord + ?Sized,
+        A::Key: Borrow<BK>,
+    {
+        let index = match target {
+            Ok(key) => A::search_key(&self.keys, key),
+            Err(Boundary::Lowest) => Err(0),
+            Err(Boundary::Highest) => Err(self.keys.len()),
+        };
+        self.remove_index(pool, index, target)
     }
 
     fn remove_index<BK>(
         &mut self,
         pool: &Pool<Node<A>>,
         index: Result<usize, usize>,
-        key: &BK,
+        target: Result<&BK, Boundary>,
     ) -> Remove<A>
     where
         A: Clone,
@@ -638,41 +763,30 @@ impl<A: BTreeValue> Node<A> {
                 match (&self.children[index], &self.children[index + 1]) {
                     // If we're a leaf, just delete the entry.
                     (&None, &None) => RemoveAction::DeleteAt(index),
-                    // Right is empty. Attempt to steal from left if enough capacity,
-                    // otherwise pull the predecessor up.
-                    (&Some(ref left), &None) => {
-                        if !left.too_small() {
-                            RemoveAction::StealFromLeft(index + 1)
-                        } else {
-                            RemoveAction::PullUp(left.keys.len() - 1, index, index)
-                        }
-                    }
-                    // Left is empty. Attempt to steal from right if enough capacity,
-                    // otherwise pull the successor up.
-                    (&None, &Some(ref right)) => {
-                        if !right.too_small() {
-                            RemoveAction::StealFromRight(index)
-                        } else {
-                            RemoveAction::PullUp(0, index, index + 1)
-                        }
-                    }
-                    // Both left and right are non empty. Attempt to steal from left or
-                    // right if enough capacity, otherwise just merge the children.
+                    // First consider pulling either predecessor (from left) or successor (from right).
+                    // otherwise just merge the two small children.
                     (&Some(ref left), &Some(ref right)) => {
-                        if left.has_room() && !right.too_small() {
-                            RemoveAction::StealFromRight(index)
-                        } else if right.has_room() && !left.too_small() {
-                            RemoveAction::StealFromLeft(index + 1)
+                        if !left.too_small() {
+                            RemoveAction::PullUp(Boundary::Highest, index, index)
+                        } else if !right.too_small() {
+                            RemoveAction::PullUp(Boundary::Lowest, index, index + 1)
                         } else {
                             RemoveAction::Merge(index)
                         }
                     }
+                    _ => unreachable!("Branch missing children"),
                 }
             }
-            // Key is adjacent to some key in node
+            // Target is adjacent to some key in node
             Err(index) => match self.children[index] {
-                // No child at location means key isn't in map.
-                None => return Remove::NoChange,
+                // We're deading with a leaf node
+                None => match target {
+                    // No child at location means key isn't in map.
+                    Ok(_key) => return Remove::NoChange,
+                    // Looking for the lowest or highest key
+                    Err(Boundary::Lowest) => RemoveAction::DeleteAt(0),
+                    Err(Boundary::Highest) => RemoveAction::DeleteAt(self.keys.len() - 1),
+                },
                 // Child at location, but it's at minimum capacity.
                 Some(ref child) if child.too_small() => {
                     let left = if index > 0 {
@@ -708,13 +822,13 @@ impl<A: BTreeValue> Node<A> {
                 self.children.remove(index);
                 Remove::Removed(pair)
             }
-            RemoveAction::PullUp(target_index, pull_to, child_index) => {
+            RemoveAction::PullUp(boundary, pull_to, child_index) => {
                 let children = &mut self.children;
                 let mut update = None;
                 let value;
                 if let Some(&mut Some(ref mut child_ref)) = children.get_mut(child_index) {
                     let child = PoolRef::make_mut(pool, child_ref);
-                    match child.remove_index(pool, Ok(target_index), key) {
+                    match child.remove_target(pool, Err(boundary)) {
                         Remove::NoChange => unreachable!(),
                         Remove::Removed(pulled_value) => {
                             value = self.keys.set(pull_to, pulled_value);
@@ -741,7 +855,7 @@ impl<A: BTreeValue> Node<A> {
                     PoolRef::unwrap_or_clone(left),
                     PoolRef::unwrap_or_clone(right),
                 );
-                let (removed, new_child) = match merged_child.remove(pool, key) {
+                let (removed, new_child) = match merged_child.remove_target(pool, target) {
                     Remove::NoChange => unreachable!(),
                     Remove::Removed(removed) => (removed, merged_child),
                     Remove::Update(removed, updated_child) => (removed, updated_child),
@@ -760,13 +874,7 @@ impl<A: BTreeValue> Node<A> {
                 {
                     let mut children = self.children.as_mut_slice()[index - 1..=index]
                         .iter_mut()
-                        .map(|n| {
-                            if let Some(ref mut o) = *n {
-                                o
-                            } else {
-                                unreachable!()
-                            }
-                        });
+                        .map(|n| n.as_mut().unwrap());
                     let left = PoolRef::make_mut(pool, children.next().unwrap());
                     let child = PoolRef::make_mut(pool, children.next().unwrap());
                     // Prepare the rebalanced node.
@@ -774,7 +882,7 @@ impl<A: BTreeValue> Node<A> {
                         left.children.last().unwrap().clone(),
                         self.keys[index - 1].clone(),
                     );
-                    match child.remove(pool, key) {
+                    match child.remove_target(pool, target) {
                         Remove::NoChange => {
                             // Key wasn't there, we need to revert the steal.
                             child.pop_min();
@@ -806,18 +914,12 @@ impl<A: BTreeValue> Node<A> {
                 {
                     let mut children = self.children.as_mut_slice()[index..index + 2]
                         .iter_mut()
-                        .map(|n| {
-                            if let Some(ref mut o) = *n {
-                                o
-                            } else {
-                                unreachable!()
-                            }
-                        });
+                        .map(|n| n.as_mut().unwrap());
                     let child = PoolRef::make_mut(pool, children.next().unwrap());
                     let right = PoolRef::make_mut(pool, children.next().unwrap());
                     // Prepare the rebalanced node.
                     child.push_max(right.children[0].clone(), self.keys[index].clone());
-                    match child.remove(pool, key) {
+                    match child.remove_target(pool, target) {
                         Remove::NoChange => {
                             // Key wasn't there, we need to revert the steal.
                             child.pop_max();
@@ -844,11 +946,17 @@ impl<A: BTreeValue> Node<A> {
                 Remove::Removed(out_value)
             }
             RemoveAction::MergeFirst(index) => {
-                if self.keys[index].cmp_keys(key) != Ordering::Equal
-                    && !self.child_contains(index, key)
-                    && !self.child_contains(index + 1, key)
-                {
-                    return Remove::NoChange;
+                if let Ok(key) = target {
+                    // Bail early if we're looking for a not existing key
+                    match self.keys[index].cmp_keys(key) {
+                        Ordering::Less if !self.child_contains(index + 1, key) => {
+                            return Remove::NoChange
+                        }
+                        Ordering::Greater if !self.child_contains(index, key) => {
+                            return Remove::NoChange
+                        }
+                        _ => (),
+                    }
                 }
                 let left = self.children.remove(index).unwrap();
                 let right = mem::replace(&mut self.children[index], None).unwrap();
@@ -860,7 +968,7 @@ impl<A: BTreeValue> Node<A> {
                 );
                 let update;
                 let out_value;
-                match merged.remove(pool, key) {
+                match merged.remove_target(pool, target) {
                     Remove::NoChange => {
                         panic!("nodes::btree::Node::remove: caught an absent key too late while merging");
                     }
@@ -887,7 +995,7 @@ impl<A: BTreeValue> Node<A> {
                 let out_value;
                 if let Some(&mut Some(ref mut child_ref)) = self.children.get_mut(index) {
                     let child = PoolRef::make_mut(pool, child_ref);
-                    match child.remove(pool, key) {
+                    match child.remove_target(pool, target) {
                         Remove::NoChange => return Remove::NoChange,
                         Remove::Removed(value) => {
                             out_value = value;
@@ -913,7 +1021,13 @@ impl<A: BTreeValue> Node<A> {
 
 /// An iterator over an ordered set.
 pub struct Iter<'a, A> {
+    /// Path to the next element that we'll yield if we take a forward step.  Each element here is
+    /// of the form `(node, index)`. For the last path element, `index` points to the next key to
+    /// yield. For every other path element, `index` is the child index of the next node in the
+    /// path.
     fwd_path: Vec<(&'a Node<A>, usize)>,
+    /// Path to the next element that we'll yield if we take a backward step. This has the same
+    /// format as `fwd_path`.
     back_path: Vec<(&'a Node<A>, usize)>,
     pub(crate) remaining: usize,
 }
@@ -1009,15 +1123,24 @@ impl<'a, A: BTreeValue> Iter<'a, A> {
     }
 
     fn step_back(path: &mut Vec<(&'a Node<A>, usize)>) -> Option<&'a A> {
+        // TODO: we're doing some repetitive leaf-vs-internal checking.
         match path.pop() {
             Some((node, index)) => match node.children[index] {
                 Some(ref child) => {
                     path.push((node, index));
-                    let mut end = child.keys.len() - 1;
+                    let mut end = if child.is_leaf() {
+                        child.keys.len() - 1
+                    } else {
+                        child.children.len() - 1
+                    };
                     path.push((child, end));
                     let mut node = child;
-                    while let Some(ref right_child) = node.children[end + 1] {
-                        end = right_child.keys.len() - 1;
+                    while let Some(ref right_child) = node.children[end] {
+                        end = if right_child.is_leaf() {
+                            right_child.keys.len() - 1
+                        } else {
+                            right_child.children.len() - 1
+                        };
                         path.push((right_child, end));
                         node = right_child;
                     }
@@ -1268,8 +1391,8 @@ impl<'a, A: 'a> DiffIter<'a, A> {
     }
 
     fn push_node(stack: &mut Vec<IterItem<'a, A>>, maybe_node: &'a Option<PoolRef<Node<A>>>) {
-        if let Some(ref node) = *maybe_node {
-            stack.push(IterItem::Consider(&node))
+        if let Some(node) = maybe_node {
+            stack.push(IterItem::Consider(node))
         }
     }
 
@@ -1294,11 +1417,11 @@ where
             match (self.old_stack.pop(), self.new_stack.pop()) {
                 (None, None) => return None,
                 (None, Some(new)) => match new {
-                    IterItem::Consider(new) => Self::push(&mut self.new_stack, &new),
+                    IterItem::Consider(new) => Self::push(&mut self.new_stack, new),
                     IterItem::Yield(new) => return Some(DiffItem::Add(new)),
                 },
                 (Some(old), None) => match old {
-                    IterItem::Consider(old) => Self::push(&mut self.old_stack, &old),
+                    IterItem::Consider(old) => Self::push(&mut self.old_stack, old),
                     IterItem::Yield(old) => return Some(DiffItem::Remove(old)),
                 },
                 (Some(old), Some(new)) => match (old, new) {
@@ -1306,29 +1429,29 @@ where
                         if !std::ptr::eq(old, new) {
                             match old.keys[0].cmp_values(&new.keys[0]) {
                                 Ordering::Less => {
-                                    Self::push(&mut self.old_stack, &old);
+                                    Self::push(&mut self.old_stack, old);
                                     self.new_stack.push(IterItem::Consider(new));
                                 }
                                 Ordering::Greater => {
                                     self.old_stack.push(IterItem::Consider(old));
-                                    Self::push(&mut self.new_stack, &new);
+                                    Self::push(&mut self.new_stack, new);
                                 }
                                 Ordering::Equal => {
-                                    Self::push(&mut self.old_stack, &old);
-                                    Self::push(&mut self.new_stack, &new);
+                                    Self::push(&mut self.old_stack, old);
+                                    Self::push(&mut self.new_stack, new);
                                 }
                             }
                         }
                     }
                     (IterItem::Consider(old), IterItem::Yield(new)) => {
-                        Self::push(&mut self.old_stack, &old);
+                        Self::push(&mut self.old_stack, old);
                         self.new_stack.push(IterItem::Yield(new));
                     }
                     (IterItem::Yield(old), IterItem::Consider(new)) => {
                         self.old_stack.push(IterItem::Yield(old));
-                        Self::push(&mut self.new_stack, &new);
+                        Self::push(&mut self.new_stack, new);
                     }
-                    (IterItem::Yield(old), IterItem::Yield(new)) => match old.cmp_values(&new) {
+                    (IterItem::Yield(old), IterItem::Yield(new)) => match old.cmp_values(new) {
                         Ordering::Less => {
                             self.new_stack.push(IterItem::Yield(new));
                             return Some(DiffItem::Remove(old));
