@@ -41,8 +41,8 @@ pub use crate::nodes::btree::{
 /// # Examples
 ///
 /// ```
-/// # #[macro_use] extern crate im;
-/// # use im::ordmap::OrdMap;
+/// # #[macro_use] extern crate imbl;
+/// # use imbl::ordmap::OrdMap;
 /// # fn main() {
 /// assert_eq!(
 ///   ordmap!{
@@ -199,8 +199,8 @@ impl<K, V> OrdMap<K, V> {
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordmap::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordmap::OrdMap;
     /// let map = OrdMap::unit(123, "onetwothree");
     /// assert_eq!(
     ///   map.get(&123),
@@ -226,8 +226,8 @@ impl<K, V> OrdMap<K, V> {
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordmap::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordmap::OrdMap;
     /// assert!(
     ///   !ordmap!{1 => 2}.is_empty()
     /// );
@@ -261,8 +261,8 @@ impl<K, V> OrdMap<K, V> {
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordmap::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordmap::OrdMap;
     /// assert_eq!(3, ordmap!{
     ///   1 => 11,
     ///   2 => 22,
@@ -294,8 +294,8 @@ impl<K, V> OrdMap<K, V> {
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::OrdMap;
     /// let mut map = ordmap![1=>1, 2=>2, 3=>3];
     /// map.clear();
     /// assert!(map.is_empty());
@@ -320,8 +320,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordmap::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordmap::OrdMap;
     /// assert_eq!(Some(&(3, 33)), ordmap!{
     ///   1 => 11,
     ///   2 => 22,
@@ -341,8 +341,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordmap::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordmap::OrdMap;
     /// assert_eq!(Some(&(1, 11)), ordmap!{
     ///   1 => 11,
     ///   2 => 22,
@@ -412,8 +412,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordmap::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordmap::OrdMap;
     /// let map = ordmap!{123 => "lol"};
     /// assert_eq!(
     ///   map.get(&123),
@@ -436,8 +436,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordmap::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordmap::OrdMap;
     /// let map = ordmap!{123 => "lol"};
     /// assert_eq!(
     ///   map.get_key_value(&123),
@@ -464,8 +464,8 @@ where
     /// # Examples
     ///
     /// ```rust
-    /// # #[macro_use] extern crate im;
-    /// # use im::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::OrdMap;
     /// let map = ordmap![1 => 1, 3 => 3, 5 => 5];
     /// assert_eq!(Some((&3, &3)), map.get_prev(&4));
     /// ```
@@ -489,8 +489,8 @@ where
     /// # Examples
     ///
     /// ```rust
-    /// # #[macro_use] extern crate im;
-    /// # use im::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::OrdMap;
     /// let map = ordmap![1 => 1, 3 => 3, 5 => 5];
     /// assert_eq!(Some((&5, &5)), map.get_next(&4));
     /// ```
@@ -510,8 +510,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordmap::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordmap::OrdMap;
     /// let map = ordmap!{123 => "lol"};
     /// assert!(
     ///   map.contains_key(&123)
@@ -572,8 +572,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordmap::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordmap::OrdMap;
     /// let map1 = ordmap!{1 => 1, 2 => 2};
     /// let map2 = ordmap!{1 => 1, 2 => 2, 3 => 3};
     /// assert!(map1.is_submap(map2));
@@ -597,8 +597,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordmap::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordmap::OrdMap;
     /// let map1 = ordmap!{1 => 1, 2 => 2};
     /// let map2 = ordmap!{1 => 1, 2 => 2, 3 => 3};
     /// assert!(map1.is_proper_submap(map2));
@@ -629,8 +629,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordmap::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordmap::OrdMap;
     /// let mut map = ordmap!{123 => "lol"};
     /// if let Some(value) = map.get_mut(&123) {
     ///     *value = "omg";
@@ -661,8 +661,8 @@ where
     /// # Examples
     ///
     /// ```rust
-    /// # #[macro_use] extern crate im;
-    /// # use im::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::OrdMap;
     /// let mut map = ordmap![1 => 1, 3 => 3, 5 => 5];
     /// if let Some((key, value)) = map.get_prev_mut(&4) {
     ///     *value = 4;
@@ -692,8 +692,8 @@ where
     /// # Examples
     ///
     /// ```rust
-    /// # #[macro_use] extern crate im;
-    /// # use im::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::OrdMap;
     /// let mut map = ordmap![1 => 1, 3 => 3, 5 => 5];
     /// if let Some((key, value)) = map.get_next_mut(&4) {
     ///     *value = 4;
@@ -726,8 +726,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordmap::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordmap::OrdMap;
     /// let mut map = ordmap!{};
     /// map.insert(123, "123");
     /// map.insert(456, "456");
@@ -766,8 +766,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordmap::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordmap::OrdMap;
     /// let mut map = ordmap!{123 => "123", 456 => "456"};
     /// map.remove(&123);
     /// map.remove(&456);
@@ -820,8 +820,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordmap::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordmap::OrdMap;
     /// let map = ordmap!{};
     /// assert_eq!(
     ///   map.update(123, "123"),
@@ -972,8 +972,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordmap::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordmap::OrdMap;
     /// let map1 = ordmap!{1 => 1, 3 => 3};
     /// let map2 = ordmap!{2 => 2, 3 => 4};
     /// let expected = ordmap!{1 => 1, 2 => 2, 3 => 3};
@@ -981,11 +981,16 @@ where
     /// ```
     #[inline]
     #[must_use]
-    pub fn union(mut self, other: Self) -> Self {
-        for (k, v) in other {
-            self.entry(k).or_insert(v);
+    pub fn union(self, other: Self) -> Self {
+        let (mut to_mutate, to_consume) = if self.len() >= other.len() {
+            (self, other)
+        } else {
+            (other, self)
+        };
+        for (k, v) in to_consume {
+            to_mutate.entry(k).or_insert(v);
         }
-        self
+        to_mutate
     }
 
     /// Construct the union of two maps, using a function to decide
@@ -1020,8 +1025,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordmap::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordmap::OrdMap;
     /// let map1 = ordmap!{1 => 1, 3 => 4};
     /// let map2 = ordmap!{2 => 2, 3 => 5};
     /// let expected = ordmap!{1 => 1, 2 => 2, 3 => 9};
@@ -1031,7 +1036,20 @@ where
     /// ));
     /// ```
     #[must_use]
-    pub fn union_with_key<F>(mut self, other: Self, mut f: F) -> Self
+    pub fn union_with_key<F>(self, other: Self, mut f: F) -> Self
+    where
+        F: FnMut(&K, V, V) -> V,
+    {
+        if self.len() >= other.len() {
+            self.union_with_key_inner(other, f)
+        } else {
+            other.union_with_key_inner(self, |key, other_value, self_value| {
+                f(key, self_value, other_value)
+            })
+        }
+    }
+
+    fn union_with_key_inner<F>(mut self, other: Self, mut f: F) -> Self
     where
         F: FnMut(&K, V, V) -> V,
     {
@@ -1057,8 +1075,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordmap::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordmap::OrdMap;
     /// let map1 = ordmap!{1 => 1, 3 => 3};
     /// let map2 = ordmap!{2 => 2};
     /// let expected = ordmap!{1 => 1, 2 => 2, 3 => 3};
@@ -1124,8 +1142,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordmap::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordmap::OrdMap;
     /// let map1 = ordmap!{1 => 1, 3 => 4};
     /// let map2 = ordmap!{2 => 2, 3 => 5};
     /// let expected = ordmap!{1 => 1, 2 => 2};
@@ -1147,8 +1165,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordmap::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordmap::OrdMap;
     /// let map1 = ordmap!{1 => 1, 3 => 4};
     /// let map2 = ordmap!{2 => 2, 3 => 5};
     /// let expected = ordmap!{1 => 1, 2 => 2};
@@ -1204,8 +1222,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordmap::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordmap::OrdMap;
     /// let map1 = ordmap!{1 => 1, 3 => 4};
     /// let map2 = ordmap!{2 => 2, 3 => 5};
     /// let expected = ordmap!{1 => 1, 2 => 2, 3 => 9};
@@ -1232,8 +1250,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordmap::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordmap::OrdMap;
     /// let map1 = ordmap!{1 => 1, 3 => 4};
     /// let map2 = ordmap!{2 => 2, 3 => 5};
     /// let expected = ordmap!{1 => 1, 2 => 2, 3 => 9};
@@ -1271,8 +1289,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordmap::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordmap::OrdMap;
     /// let map1 = ordmap!{1 => 1, 3 => 4};
     /// let map2 = ordmap!{2 => 2, 3 => 5};
     /// let expected = ordmap!{1 => 1};
@@ -1295,8 +1313,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordmap::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordmap::OrdMap;
     /// let map1 = ordmap!{1 => 1, 2 => 2};
     /// let map2 = ordmap!{2 => 3, 3 => 4};
     /// let expected = ordmap!{2 => 2};
@@ -1333,8 +1351,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordmap::OrdMap;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordmap::OrdMap;
     /// let map1 = ordmap!{1 => 1, 2 => 2};
     /// let map2 = ordmap!{2 => 3, 3 => 4};
     /// let expected = ordmap!{2 => 5};
@@ -2163,7 +2181,7 @@ impl<'a, K: Ord + Hash + Eq + Clone, V: Clone, S: BuildHasher> From<&'a HashMap<
 pub mod proptest {
     #[deprecated(
         since = "14.3.0",
-        note = "proptest strategies have moved to im::proptest"
+        note = "proptest strategies have moved to imbl::proptest"
     )]
     pub use crate::proptest::ord_map;
 }
@@ -2381,6 +2399,30 @@ mod test {
         assert_eq!(vec![(1, 2), (2, 3), (3, 4), (4, 5), (5, 6)], range);
     }
 
+    #[test]
+    fn range_iter_big() {
+        use crate::nodes::btree::NODE_SIZE;
+        use std::ops::Bound::Included;
+        const N: usize = NODE_SIZE * NODE_SIZE * 5; // enough for a sizeable 3 level tree
+
+        let data = (1usize..N).filter(|i| i % 2 == 0).map(|i| (i, ()));
+        let bmap = data
+            .clone()
+            .collect::<std::collections::BTreeMap<usize, ()>>();
+        let omap = data.collect::<OrdMap<usize, ()>>();
+
+        for i in (0..NODE_SIZE * 5).chain(N - NODE_SIZE * 5..=N + 1) {
+            assert_eq!(omap.range(i..).count(), bmap.range(i..).count());
+            assert_eq!(omap.range(..i).count(), bmap.range(..i).count());
+            assert_eq!(omap.range(i..i + 7).count(), bmap.range(i..i + 7).count());
+            assert_eq!(omap.range(i..=i + 7).count(), bmap.range(i..=i + 7).count());
+            assert_eq!(
+                omap.range((Included(i), Included(i + 7))).count(),
+                bmap.range((Included(i), Included(i + 7))).count(),
+            );
+        }
+    }
+
     #[test]
     fn issue_124() {
         let mut map = OrdMap::new();
