@@ -7,15 +7,13 @@
 //! An immutable ordered set implemented as a [B-tree] [1].
 //!
 //! Most operations on this type of set are O(log n). A
-//! [`HashSet`][hashset::HashSet] is usually a better choice for
+//! [`HashSet`] is usually a better choice for
 //! performance, but the `OrdSet` has the advantage of only requiring
 //! an [`Ord`][std::cmp::Ord] constraint on its values, and of being
 //! ordered, so values always come out from lowest to highest, where a
-//! [`HashSet`][hashset::HashSet] has no guaranteed ordering.
+//! [`HashSet`] has no guaranteed ordering.
 //!
 //! [1]: https://en.wikipedia.org/wiki/B-tree
-//! [hashset::HashSet]: ../hashset/struct.HashSet.html
-//! [std::cmp::Ord]: https://doc.rust-lang.org/std/cmp/trait.Ord.html
 
 use std::borrow::Borrow;
 use std::cmp::Ordering;
@@ -41,8 +39,8 @@ pub use crate::nodes::btree::DiffItem;
 /// # Examples
 ///
 /// ```
-/// # #[macro_use] extern crate im;
-/// # use im::ordset::OrdSet;
+/// # #[macro_use] extern crate imbl;
+/// # use imbl::ordset::OrdSet;
 /// # fn main() {
 /// assert_eq!(
 ///   ordset![1, 2, 3],
@@ -163,15 +161,13 @@ def_pool!(OrdSetPool<A>, Node<Value<A>>);
 /// An immutable ordered set implemented as a [B-tree] [1].
 ///
 /// Most operations on this type of set are O(log n). A
-/// [`HashSet`][hashset::HashSet] is usually a better choice for
+/// [`HashSet`] is usually a better choice for
 /// performance, but the `OrdSet` has the advantage of only requiring
 /// an [`Ord`][std::cmp::Ord] constraint on its values, and of being
 /// ordered, so values always come out from lowest to highest, where a
-/// [`HashSet`][hashset::HashSet] has no guaranteed ordering.
+/// [`HashSet`] has no guaranteed ordering.
 ///
 /// [1]: https://en.wikipedia.org/wiki/B-tree
-/// [hashset::HashSet]: ../hashset/struct.HashSet.html
-/// [std::cmp::Ord]: https://doc.rust-lang.org/std/cmp/trait.Ord.html
 pub struct OrdSet<A> {
     size: usize,
     pool: OrdSetPool<A>,
@@ -208,8 +204,8 @@ impl<A> OrdSet<A> {
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordset::OrdSet;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordset::OrdSet;
     /// let set = OrdSet::unit(123);
     /// assert!(set.contains(&123));
     /// ```
@@ -232,8 +228,8 @@ impl<A> OrdSet<A> {
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordset::OrdSet;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordset::OrdSet;
     /// assert!(
     ///   !ordset![1, 2, 3].is_empty()
     /// );
@@ -254,8 +250,8 @@ impl<A> OrdSet<A> {
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordset::OrdSet;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordset::OrdSet;
     /// assert_eq!(3, ordset![1, 2, 3].len());
     /// ```
     #[inline]
@@ -296,8 +292,8 @@ impl<A> OrdSet<A> {
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::OrdSet;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::OrdSet;
     /// let mut set = ordset![1, 2, 3];
     /// set.clear();
     /// assert!(set.is_empty());
@@ -380,8 +376,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordset::OrdSet;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordset::OrdSet;
     /// let mut set = ordset!{1, 2, 3};
     /// assert!(set.contains(&1));
     /// assert!(!set.contains(&4));
@@ -406,8 +402,8 @@ where
     /// # Examples
     ///
     /// ```rust
-    /// # #[macro_use] extern crate im;
-    /// # use im::OrdSet;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::OrdSet;
     /// let set = ordset![1, 3, 5, 7, 9];
     /// assert_eq!(Some(&5), set.get_prev(&6));
     /// ```
@@ -426,8 +422,8 @@ where
     /// # Examples
     ///
     /// ```rust
-    /// # #[macro_use] extern crate im;
-    /// # use im::OrdSet;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::OrdSet;
     /// let set = ordset![1, 3, 5, 7, 9];
     /// assert_eq!(Some(&5), set.get_next(&4));
     /// ```
@@ -449,7 +445,7 @@ where
         if other.len() < self.len() {
             return false;
         }
-        self.iter().all(|a| other.contains(&a))
+        self.iter().all(|a| other.contains(a))
     }
 
     /// Test whether a set is a proper subset of another set, meaning
@@ -477,8 +473,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordset::OrdSet;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordset::OrdSet;
     /// let mut set = ordset!{};
     /// set.insert(123);
     /// set.insert(456);
@@ -567,8 +563,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordset::OrdSet;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordset::OrdSet;
     /// let set = ordset![456];
     /// assert_eq!(
     ///   set.update(123),
@@ -604,7 +600,7 @@ where
     #[must_use]
     pub fn without_min(&self) -> (Option<A>, Self) {
         match self.get_min() {
-            Some(v) => (Some(v.clone()), self.without(&v)),
+            Some(v) => (Some(v.clone()), self.without(v)),
             None => (None, self.clone()),
         }
     }
@@ -616,7 +612,7 @@ where
     #[must_use]
     pub fn without_max(&self) -> (Option<A>, Self) {
         match self.get_max() {
-            Some(v) => (Some(v.clone()), self.without(&v)),
+            Some(v) => (Some(v.clone()), self.without(v)),
             None => (None, self.clone()),
         }
     }
@@ -628,19 +624,24 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordset::OrdSet;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordset::OrdSet;
     /// let set1 = ordset!{1, 2};
     /// let set2 = ordset!{2, 3};
     /// let expected = ordset!{1, 2, 3};
     /// assert_eq!(expected, set1.union(set2));
     /// ```
     #[must_use]
-    pub fn union(mut self, other: Self) -> Self {
-        for value in other {
-            self.insert(value);
+    pub fn union(self, other: Self) -> Self {
+        let (mut to_mutate, to_consume) = if self.len() >= other.len() {
+            (self, other)
+        } else {
+            (other, self)
+        };
+        for value in to_consume {
+            to_mutate.insert(value);
         }
-        self
+        to_mutate
     }
 
     /// Construct the union of multiple sets.
@@ -664,8 +665,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordset::OrdSet;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordset::OrdSet;
     /// let set1 = ordset!{1, 2};
     /// let set2 = ordset!{2, 3};
     /// let expected = ordset!{1, 3};
@@ -685,8 +686,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordset::OrdSet;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordset::OrdSet;
     /// let set1 = ordset!{1, 2};
     /// let set2 = ordset!{2, 3};
     /// let expected = ordset!{1, 3};
@@ -710,8 +711,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordset::OrdSet;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordset::OrdSet;
     /// let set1 = ordset!{1, 2};
     /// let set2 = ordset!{2, 3};
     /// let expected = ordset!{1};
@@ -732,8 +733,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[macro_use] extern crate im;
-    /// # use im::ordset::OrdSet;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::ordset::OrdSet;
     /// let set1 = ordset!{1, 2};
     /// let set2 = ordset!{2, 3};
     /// let expected = ordset!{2};
@@ -1169,7 +1170,7 @@ impl<'a, A: Hash + Eq + Ord + Clone, S: BuildHasher> From<&'a HashSet<A, S>> for
 pub mod proptest {
     #[deprecated(
         since = "14.3.0",
-        note = "proptest strategies have moved to im::proptest"
+        note = "proptest strategies have moved to imbl::proptest"
     )]
     pub use crate::proptest::ord_set;
 }
@@ -1219,6 +1220,7 @@ mod test {
         fn proptest_a_set(ref s in ord_set(".*", 10..100)) {
             assert!(s.len() < 100);
             assert!(s.len() >= 10);
+            s.root.check_sane();
         }
 
         #[test]
@@ -1226,11 +1228,13 @@ mod test {
             let range = 0..max;
             let expected: Vec<i32> = range.clone().collect();
             let set: OrdSet<i32> = OrdSet::from_iter(range.clone());
+            set.root.check_sane();
             let result: Vec<i32> = set.range(..).cloned().collect();
             assert_eq!(expected, result);
 
             let expected: Vec<i32> = range.clone().rev().collect();
             let set: OrdSet<i32> = OrdSet::from_iter(range);
+            set.root.check_sane();
             let result: Vec<i32> = set.range(..).rev().cloned().collect();
             assert_eq!(expected, result);
         }
