@@ -54,8 +54,8 @@ use crate::vector::{
 /// the scenes, so the performance of the two should be identical.
 ///
 /// ```rust
-/// # #[macro_use] extern crate im;
-/// # use im::vector::Vector;
+/// # #[macro_use] extern crate imbl;
+/// # use imbl::vector::Vector;
 /// # use std::iter::FromIterator;
 /// let mut vec: Vector<i64> = Vector::from_iter(0..1000);
 ///
@@ -162,8 +162,8 @@ where
     /// ## Examples
     ///
     /// ```rust
-    /// # #[macro_use] extern crate im;
-    /// # use im::vector::Vector;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::vector::Vector;
     /// # use std::iter::FromIterator;
     /// let vec = Vector::from_iter(0..1000);
     /// let narrowed = vec.focus().narrow(100..200);
@@ -202,8 +202,8 @@ where
     /// ## Examples
     ///
     /// ```rust
-    /// # #[macro_use] extern crate im;
-    /// # use im::vector::Vector;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::vector::Vector;
     /// # use std::iter::FromIterator;
     /// let vec = Vector::from_iter(0..1000);
     /// let (left, right) = vec.focus().split_at(500);
@@ -279,10 +279,10 @@ impl<A> Clone for TreeFocus<A> {
 
 #[allow(unsafe_code)]
 #[cfg(threadsafe)]
-unsafe impl<A> Send for TreeFocus<A> {}
+unsafe impl<A: Send> Send for TreeFocus<A> {}
 #[allow(unsafe_code)]
 #[cfg(threadsafe)]
-unsafe impl<A> Sync for TreeFocus<A> {}
+unsafe impl<A: Sync> Sync for TreeFocus<A> {}
 
 #[inline]
 fn contains<A: Ord>(range: &Range<A>, index: &A) -> bool {
@@ -414,8 +414,8 @@ where
 /// on the structure of the vector not changing while it exists.
 ///
 /// ```rust,compile_fail
-/// # #[macro_use] extern crate im;
-/// # use im::vector::Vector;
+/// # #[macro_use] extern crate imbl;
+/// # use imbl::vector::Vector;
 /// # use std::iter::FromIterator;
 /// let mut vec = Vector::from_iter(0..1000);
 /// let focus1 = vec.focus_mut();
@@ -431,8 +431,8 @@ where
 /// which is safe because they can't overlap:
 ///
 /// ```rust
-/// # #[macro_use] extern crate im;
-/// # use im::vector::Vector;
+/// # #[macro_use] extern crate imbl;
+/// # use imbl::vector::Vector;
 /// # use std::iter::FromIterator;
 /// let mut vec = Vector::from_iter(0..1000);
 /// let focus = vec.focus_mut();
@@ -445,8 +445,8 @@ where
 /// were created from goes out of scope.
 ///
 /// ```rust,compile_fail
-/// # #[macro_use] extern crate im;
-/// # use im::vector::Vector;
+/// # #[macro_use] extern crate imbl;
+/// # use imbl::vector::Vector;
 /// # use std::iter::FromIterator;
 /// let mut vec = Vector::from_iter(0..1000);
 /// let (left, right) = {
@@ -529,10 +529,7 @@ where
     /// Returns `None` if the index is out of bounds, or the replaced value
     /// otherwise.
     pub fn set(&mut self, index: usize, value: A) -> Option<A> {
-        match self.get_mut(index) {
-            Some(ref mut pos) => Some(replace(pos, value)),
-            None => None,
-        }
+        self.get_mut(index).map(|pos| replace(pos, value))
     }
 
     /// Swap the values at two given indices.
@@ -557,8 +554,8 @@ where
     /// # Examples
     ///
     /// ```rust
-    /// # #[macro_use] extern crate im;
-    /// # use im::vector::Vector;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::vector::Vector;
     /// # use std::iter::FromIterator;
     /// let mut vec = vector![1, 2, 3, 4, 5];
     /// vec.focus_mut().pair(1, 3, |a, b| *a += *b);
@@ -587,8 +584,8 @@ where
     /// # Examples
     ///
     /// ```rust
-    /// # #[macro_use] extern crate im;
-    /// # use im::vector::Vector;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::vector::Vector;
     /// # use std::iter::FromIterator;
     /// let mut vec = vector![1, 2, 3, 4, 5];
     /// vec.focus_mut().triplet(0, 2, 4, |a, b, c| *a += *b + *c);
@@ -636,8 +633,8 @@ where
     /// ## Examples
     ///
     /// ```rust
-    /// # #[macro_use] extern crate im;
-    /// # use im::vector::Vector;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::vector::Vector;
     /// # use std::iter::FromIterator;
     /// let mut vec = Vector::from_iter(0..1000);
     /// let narrowed = vec.focus_mut().narrow(100..200);
@@ -676,8 +673,8 @@ where
     /// ## Examples
     ///
     /// ```rust
-    /// # #[macro_use] extern crate im;
-    /// # use im::vector::Vector;
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::vector::Vector;
     /// # use std::iter::FromIterator;
     /// let mut vec = Vector::from_iter(0..1000);
     /// {
@@ -749,12 +746,12 @@ where
     }
 }
 
-impl<'a, A> Into<Focus<'a, A>> for FocusMut<'a, A>
+impl<'a, A> From<FocusMut<'a, A>> for Focus<'a, A>
 where
     A: Clone + 'a,
 {
-    fn into(self) -> Focus<'a, A> {
-        self.unmut()
+    fn from(f: FocusMut<'a, A>) -> Focus<'a, A> {
+        f.unmut()
     }
 }
 
@@ -832,7 +829,7 @@ where
         let mut tree = self
             .tree
             .lock()
-            .expect("im::vector::Focus::set_focus: unable to acquire exclusive lock on Vector");
+            .expect("imbl::vector::Focus::set_focus: unable to acquire exclusive lock on Vector");
         if index < self.middle_range.start {
             let outer_len = tree.outer_f.len();
             if index < outer_len {
